import karamba
import os
import sys
import string
import re
import threading
import urllib2

ping = "216.239.37.99" # google.com
iface = "eth0"
maxkBs = 128

pingPID = -1
pingOptionPID = -1
ifaceOptionPID = -1
maxkBsOptionPID = -1

natIpUpdate = 0

def shell2string(shell):
    (pipein, pipe) = os.popen4(shell, "r")
    pipein.close()
    ret = pipe.readlines()
    pipe.close()
    return ret

def readConfigEntryWithDefault(widget, key, default):
    value = karamba.readConfigEntry(widget, key)
    if str(value) != "None":
	return value
    else:
        return default    

def initWidget(widget):
    global ping
    ping = readConfigEntryWithDefault(widget, "ping", ping)

    global iface
    iface = readConfigEntryWithDefault(widget, "iface", iface)

    global maxkBs
    maxkBs = int(readConfigEntryWithDefault(widget, "maxkBs", maxkBs))
    setFlowGraphMax(widget, maxkBs)

    karamba.addMenuConfigOption(widget, "iface", "Interface")
    karamba.setMenuConfigOption(widget, "iface", 0)
    karamba.addMenuConfigOption(widget, "maxkBs", "Max kB/s")
    karamba.setMenuConfigOption(widget, "maxkBs", 0)
    karamba.addMenuConfigOption(widget, "ping", "Ping IP")
    karamba.setMenuConfigOption(widget, "ping", 0)
    widgetUpdated(widget)


def netstatParser():
    ip = "[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+"
    port = "[*0-9a-z]+"
    netstat = shell2string("netstat -pa --numeric-hosts --inet")
    server = []
    client = {}
    for line in netstat:
        catch = re.findall("^[a-z]+ +[0-9]+ +[0-9]+ +"+ip+":("+port+") +"+ip+":("+port+") +([A-Z]+) +([^ \n]+).*", line)
        if catch != []:
            local = catch[0][0]
            foreign = catch[0][1]
            state = catch[0][2]
            prog = catch[0][3]
            if state == "LISTEN":
                # server
                if re.match("[a-z]+",local) or prog == "-":
                    # named proto
                    name = local
                else:
                    name = re.findall("[0-9]+\/(.*)", prog)[0]
                server += [name]
            else:
                # client
                if prog != "-":
                    prog = re.sub("konqueror.*", "konqueror", prog)
                    prog = re.sub("kmail.*", "kmail", prog)
                    prog = re.sub("kontact.*", "kontact", prog)
                    prog = re.sub("kded.*", "kded", prog)
                    prog = re.sub("kget.*", "kget", prog)
                    prog = re.findall("[0-9]+\/(.*)", prog)[0]
                    try:
                	client[prog] += [foreign]
		    except:
			client[prog] = [foreign]
    server = set(server)
    for cli in client:
        client[cli] = set(client[cli])
    return [server,client]

def netstat(widget):
    textServer = karamba.getThemeText(widget, "server")
    textClient = karamba.getThemeText(widget, "client")
    stat = netstatParser()
    stringServer = ""
    stringClient = ""
    for server in stat[0]:
	stringServer += server + "\n"
    for cli in stat[1]:
	stringClient += cli + " ("
        for proto in stat[1][cli]:
	    stringClient += proto + ","
	stringClient = stringClient[:-1] + ")\n"
    karamba.changeText(widget, textServer, stringServer)
    karamba.changeText(widget, textClient, stringClient)

def netIp(widget, iface):
    try:
	if iface == "nat":
	    global natIpUpdate
	    natIpUpdate = (natIpUpdate+1)%10 # Update the IP only N widget update, the nat IP is long to update and use internet
	    if natIpUpdate == 1:
		thread = noLockGetNatIP(widget)
		thread.start()
	    ip = ""
	else:
    	    ifconfig = shell2string("/sbin/ifconfig " + iface )
    	    ifconfig = ifconfig[1]
    	    if iface[:3] == "ppp":
        	ip = re.sub(".*:([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}) .*", "\\1", ifconfig)
    	    else:
        	ip = re.sub(" *inet addr:([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}).*", "\\1", ifconfig)
    	    if ip == ifconfig or ip == "":
        	ip = "No IP"
    except:
        ip = "Down"
    return ip

def netPing(widget, host):
    global pingPID
    pingPID = karamba.executeInteractive(widget, ["ping", "-c1", "-W2", host])


def net(widget):
    global iface
    global ping
    textIp = karamba.getThemeText(widget, "netIp")
    ip = netIp(widget, iface)
    if ip != "":
	karamba.changeText(widget, textIp, ip)
    netPing(widget, ping)

def widgetUpdated(widget):
    netstat(widget)
    net(widget)

def menuOptionChanged(widget, key, value):
    if key == "ping":
	global pingOptionPID
	pingOptionPID = karamba.executeInteractive(widget, ["kdialog", "--inputbox", 'Enter ping IP (google.com:216.239.37.99)', ping])
    elif key == "iface":
	global ifaceOptionPID
	ifaceOptionPID = karamba.executeInteractive(widget, ["kdialog", "--inputbox", 'Enter the net interface to watch (eg: eth0, ppp0, adsl0)\nor "nat" if your local ip address is hidden from internet', iface])
    elif key == "maxkBs":
	global maxkBsOptionPID
	maxkBsOptionPID = karamba.executeInteractive(widget, ["kdialog", "--inputbox", 'Enter your maximun net flow in kB/s', str(maxkBs)])

def commandOutput(widget, pid, output):
    if output != "\n":
        output = string.strip(output)
	global pingPID
	global pingOptionPID
	global ifaceOptionPID
	global maxkBsOptionPID
	if pid == pingPID:
	    pingPID = -1
	    try:
		found = re.findall(".*time=([0-9]+).*", output)
		found = found[0]
		textPing = karamba.getThemeText(widget, "ping")
		karamba.changeText(widget, textPing, found+" ms")
		textPingG = karamba.getThemeGraph(widget, "pingG")
		karamba.setGraphValue(widget, textPingG, int(found))
		karamba.redrawWidget(widget)
	    except:
		textPing = karamba.getThemeText(widget, "ping")
		karamba.changeText(widget, textPing, "> 2000 ms")
		textPingG = karamba.getThemeGraph(widget, "pingG")
		karamba.setGraphValue(widget, textPingG, 10000)
	elif pid == pingOptionPID:
	    pingOptionPID = -1
    	    karamba.setMenuConfigOption(widget, "ping", 0)
	    try:
		output = re.findall("^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$", output)
		global ping
		ping = output[0]
		karamba.writeConfigEntry(widget, "ping", ping)
	    except:
		karamba.executeInteractive(widget, ["kdialog", "--error", '"'+output+'" Isn\'t a valid IP.'])
	elif pid == ifaceOptionPID:
	    ifaceOptionPID = -1
    	    karamba.setMenuConfigOption(widget, "iface", 0)
	    try:
		output = re.findall("^[a-z]+[0-9]+$|^nat$|^lo$", output)
		global iface
		iface = output[0]
    	        karamba.writeConfigEntry(widget, "iface", iface)
	        widgetUpdated(widget)
	    except:
		karamba.executeInteractive(widget, ["kdialog", "--error", '"'+output+'" Isn\'t a valid interface name.'])
	elif pid == maxkBsOptionPID:
	    maxkBsOptionPID = -1
    	    karamba.setMenuConfigOption(widget, "maxkBs", 0)
	    try:
		output = int(output)
		if output <= 0:
		    raise Exception
		global maxkBs
		maxkBs = output
    	        karamba.writeConfigEntry(widget, "maxkBs", str(maxkBs))
		setFlowGraphMax(widget, maxkBs)
	    except:
		karamba.executeInteractive(widget, ["kdialog", "--error", '"'+output+'" Isn\'t a valid flow number.'])

def setFlowGraphMax(widget, max):
    flowIn = karamba.getThemeGraph(widget, "flowIn")
    flowOut = karamba.getThemeGraph(widget, "flowOut")
    karamba.setGraphMinMax(widget, flowIn, 0, max)
    karamba.setGraphMinMax(widget, flowOut, 0, max)

class noLockGetNatIP(threading.Thread):
    def __init__(self, widget):
        threading.Thread.__init__(self)
        self.widget = widget

    def run(self):
	url = "http://checkip.dyndns.org/index.html"
        request = urllib2.Request(url)
        request.add_header("User-Agent", "r-karamba")
        opener = urllib2.build_opener()
        try:
    	    contents = opener.open(request).readlines()
	    contents = re.findall(".* ([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}).*", contents[0])
	    ip = contents[0]
	    opener.close()
        except:
	    ip = "Down-"
	textIp = karamba.getThemeText(self.widget, "netIp")
	karamba.changeText(self.widget, textIp, ip)
